==> minreact-scripts-0.0.26.tgz: package/webpack/webpack.config.js <==
const path = require("path");
const fs = require("fs");

// general plugins...
const webpack = require("webpack");
const HtmlWebpackPlugin = require("html-webpack-plugin");

// production plugins...
const MiniCssExtractPlugin = require("mini-css-extract-plugin");
const UglifyJsPlugin = require("uglifyjs-webpack-plugin");
const OptimizeCSSAssetsPlugin = require("optimize-css-assets-webpack-plugin");
const CleanWebpackPlugin = require("clean-webpack-plugin");
// const HtmlWebpackInlineSourcePlugin = require("html-webpack-inline-source-plugin");
const { BundleAnalyzerPlugin } = require("webpack-bundle-analyzer");

const basePath = process.cwd();

const projectInfo = require(path.resolve(basePath, "./package.json"));
// console.log(projectInfo)

let i = 0;

module.exports = (env = {}) => {
  const NODE_ENV = process.env.NODE_ENV || "development";
  const prodMode = NODE_ENV === "production";
  const ifAnalyze = !!env.analyze;
  const aliasObject = projectInfo.alias
    ? Object.keys(projectInfo.alias).reduce((prev, next) => {
        prev[next] = path.resolve(basePath, projectInfo.alias[next]);
        return prev;
      }, {})
    : {};
  return {
    // specify webpack runtime dir context(typically process.cwd())
    context: basePath,
    entry: "./src/index",
    output: {
      // config.output.path has to be absolute!
      path: path.resolve(basePath, "./output"),
      filename: `app${prodMode ? ".[contenthash:8]" : ""}.js`,
      chunkFilename: `[name]${prodMode ? ".[contenthash:8]" : ""}.chunk.js`,
      publicPath: prodMode ? projectInfo.home || "" : ""
    },
    resolve: {
      extensions: [".js", ".jsx", ".ts", ".tsx"],
      alias: aliasObject
    },
    // always generate source-map
    devtool: "source-map",
    // for browsers
    target: "web",
    devServer: {
      // fallback to ./src
      contentBase: "./src",
      compress: false,
      port: 8895,
      headers: { "Access-Control-Allow-Origin": "*" },
      // to-do: proxy
      proxy: projectInfo.proxy || {}
    },
    mode: NODE_ENV,
    module: {
      rules: [
        // All output '.js' files will  re-processed by 'source-map-loader' and 'eslint-loader'.
        {
          enforce: "pre",
          test: /\.jsx?$/,
          use: [
            "source-map-loader",
            {
              loader: "eslint-loader",
              options: {
                emitWarning: true,
                failOnWarning: false,
                // emitError: true,
                failOnError: true,
                configFile: path.resolve(__dirname, "./config/.eslintrc"),
                useEslintrc: false
              }
            }
          ],
          exclude: /(node_modules|bower_components)/
        },
        // for NOT css/less modules
        {
          test: /[^.module]\.(css|less)$/,
          use: [
            {
              loader: prodMode ? MiniCssExtractPlugin.loader : "style-loader"
            },
            {
              loader: "css-loader"
            },
            {
              loader: "postcss-loader",
              options: {
                ident: "postcss",
                plugins: [
                  require("autoprefixer")({
                    add: prodMode,
                    // react doesn't support ie < 11 anyway, so why css?
                    // 和babelrc中保持一致
                    browsers: [
                      ">0.25%",
                      "not dead",
                      "not ie <= 11",
                      "not op_mini all"
                    ]
                  })
                ]
              }
            },
            {
              loader: "less-loader",
              options: {
                // antd modify colors
                modifyVars: {
                  "primary-color": "#ff4852",
                  "link-color": "#ff4852",
                  "border-radius-base": "2px"
                },
                javascriptEnabled: true
              }
            }
          ]
        },
        // for css/less modules
        {
          test: /\.module\.(css|less)$/,
          use: [
            {
              loader: prodMode ? MiniCssExtractPlugin.loader : "style-loader"
            },
            {
              loader: "css-loader",
              options: {
                modules: true
              }
            },
            {
              loader: "postcss-loader",
              options: {
                ident: "postcss",
                plugins: [
                  require("autoprefixer")({
                    add: prodMode,
                    // react doesn't support ie < 11 anyway, so why css?
                    browsers: [
                      ">0.25%",
                      "not dead",
                      "not ie <= 11",
                      "not op_mini all"
                    ]
                  })
                ]
              }
            },
            {
              loader: "less-loader"
            }
          ]
        },
        {
          test: /\.(sa|sc)ss$/,
          use: [
            {
              loader: prodMode ? MiniCssExtractPlugin.loader : "style-loader"
            },
            {
              loader: "css-loader"
            },
            {
              loader: "postcss-loader",
              options: {
                ident: "postcss",
                plugins: [
                  require("autoprefixer")({
                    add: prodMode,
                    // react doesn't support ie < 11 anyway, so why css?
                    browsers: [
                      ">0.25%",
                      "not dead",
                      "not ie <= 11",
                      "not op_mini all"
                    ]
                  })
                ]
              }
            },
            {
              loader: "sass-loader"
            }
          ]
        },
        // All files with a '.ts' or '.tsx' extension will be handled by 'awesome-typescript-loader'.
        {
          test: /\.tsx?$/,
          use: [
            {
              loader: "awesome-typescript-loader",
              options: {
                configFileName: path.resolve(
                  __dirname,
                  "./config/tsconfig.json"
                )
              }
            }
          ]
        },
        // normal babel-loader
        {
          test: /\.(js|jsx)$/,
          exclude: prodMode
            ? /__invalidFolder/
            : /(node_modules|bower_components)/,
          loader: [
            {
              loader: "babel-loader",
              options: require(path.resolve(__dirname, "./config/.babelrc.js"))
            }
          ]
        },
        {
          test: /\.(jpg|gif|woff|woff2|eot|ttf|png|svg)/,
          use: [
            {
              loader: "url-loader",
              options: {
                // or handled by webpack as normal pic/asset
                limit: 100000
              }
            }
          ]
        }
        // {
        //   test: /\.svg/,
        //   use: [
        //     {
        //       loader: "url-loader",
        //       options: {
        //         limit: 100000,
        //         minetype: "image/svg+xml"
        //       }
        //     }
        //   ]
        // }
      ]
    },
    plugins: [
      new webpack.DefinePlugin({
        "process.env": {
          NODE_ENV: JSON.stringify(NODE_ENV)
        }
      }),
      new HtmlWebpackPlugin({
        filename: "index.html",
        template: path.resolve(basePath, "./index.ejs"),
        minify: {
          removeAttributeQuotes: prodMode,
          removeComments: prodMode,
          // minifyCSS: prodMode, // already minified
          minifyJS: prodMode
        },
        chunks: "all",
        // inlineSource: prodMode ? ".css$" : "",
        environment: NODE_ENV
      })
    ].concat(
      prodMode
        ? [
            new CleanWebpackPlugin(path.resolve(basePath, "./output"), {
              allowExternal: true
            }),
            new MiniCssExtractPlugin({
              filename: prodMode ? "app.[hash:8].css" : "[name].css",
              chunkFilename: prodMode ? "[id].[hash:8].css" : "[id].css"
            }),
            // will conflict with something...
            // ------------------------------------
            // Long Term Caching
            // ------------------------------------
            // More information https://medium.com/webpack/predictable-long-term-caching-with-webpack-d3eee1d3fa31
            new webpack.NamedChunksPlugin(chunk => {
              if (chunk.name) {
                return chunk.name;
              } else {
                // console.log(chunk);
                // debugger;
                // return String(++i);
                // 原来的代码会报错, 这里先hack一下...
                return [...chunk._modules][0].context.replace(/^\/.*\//, "");
              }

              // eslint-disable-next-line no-underscore-dangle
              // return [...chunk._modules]
              //   .map(m =>
              //     path.relative(
              //       m.context,
              //       m.userRequest.substring(0, m.userRequest.lastIndexOf('.'))
              //     )
              //   )
              //   .join('_');
            }),
            new webpack.HashedModuleIdsPlugin(),
            // ------------------------------------
            // Inject resulting css file into html...(optional)
            // ------------------------------------
            // new HtmlWebpackInlineSourcePlugin(),
            ifAnalyze
              ? new BundleAnalyzerPlugin({
                  port: "9981"
                })
              : () => {}
            // new StyleExtHtmlWebpackPlugin()
            // new CopyWebpackPlugin([
            //   {
            //     from: path.join(__dirname, './public'),
            //     to: path.join(__dirname, 'output/public')
            //   }
            // ])
          ]
        : []
    ),
    optimization: Object.assign(
      {
        splitChunks: {
          cacheGroups: {
            // ------------------------------------
            // this is for multi-entries only
            // ------------------------------------
            // commons: {
            //   name: 'commons',
            //   chunks: 'initial',
            //   minChunks: 2
            // },
            // ------------------------------------
            // vendors chunk
            // ------------------------------------
            vendors: {
              test(module) {
                // debugger
                // console.log(module, chunks);
                return (
                  /[\\/]node_modules[\\/]/.test(module.context) &&
                  !/[\\/]node_modules[\\/](react|react-dom)[\\/]/.test(
                    module.context
                  )
                );
              },
              name: "vendors",
              chunks: "all"
            },
            // ------------------------------------
            // react and react-dom chunk
            // ------------------------------------
            react: {
              test(module) {
                return /[\\/]node_modules[\\/](react|react-dom)[\\/]/.test(
                  module.context
                );
              },
              name: "react",
              chunks: "all"
            }
          }
          // styles: {
          //   name: 'styles',
          //   test: /\.css$/,
          //   chunks: 'all',
          //   enforce: true
          // }
        }
      },
      prodMode
        ? {
            minimizer: [
              new UglifyJsPlugin({
                cache: true,
                parallel: true,
                sourceMap: true // set to true if you want JS source maps
              }),
              new OptimizeCSSAssetsPlugin({})
            ]
          }
        : {}
    )
  };
};

==> minreact-scripts-0.0.27.tgz: package/webpack/webpack.config.js <==
const path = require("path");
const fs = require("fs");

// general plugins...
const webpack = require("webpack");
const HtmlWebpackPlugin = require("html-webpack-plugin");

// production plugins...
const MiniCssExtractPlugin = require("mini-css-extract-plugin");
const UglifyJsPlugin = require("uglifyjs-webpack-plugin");
const OptimizeCSSAssetsPlugin = require("optimize-css-assets-webpack-plugin");
const CleanWebpackPlugin = require("clean-webpack-plugin");
// const HtmlWebpackInlineSourcePlugin = require("html-webpack-inline-source-plugin");
const { BundleAnalyzerPlugin } = require("webpack-bundle-analyzer");

const basePath = process.cwd();

const projectInfo = require(path.resolve(basePath, "./package.json"));
// console.log(projectInfo)

let i = 0;

module.exports = (env = {}) => {
  const NODE_ENV = process.env.NODE_ENV || "development";
  const prodMode = NODE_ENV === "production";
  const ifAnalyze = !!env.analyze;
  const aliasObject = projectInfo.alias
    ? Object.keys(projectInfo.alias).reduce((prev, next) => {
        prev[next] = path.resolve(basePath, projectInfo.alias[next]);
        return prev;
      }, {})
    : {};
  const externalTsConfigExist = fs.existsSync(
    path.join(basePath, "./tsconfig.json")
  );
  return {
    // specify webpack runtime dir context(typically process.cwd())
    context: basePath,
    entry: "./src/index",
    output: {
      // config.output.path has to be absolute!
      path: path.resolve(basePath, "./output"),
      filename: `app${prodMode ? ".[contenthash:8]" : ""}.js`,
      chunkFilename: `[name]${prodMode ? ".[contenthash:8]" : ""}.chunk.js`,
      publicPath: prodMode ? projectInfo.home || "" : ""
    },
    resolve: {
      extensions: [".js", ".jsx", ".ts", ".tsx"],
      alias: aliasObject
    },
    // always generate source-map
    devtool: "source-map",
    // for browsers
    target: "web",
    devServer: {
      // fallback to ./src
      contentBase: "./src",
      compress: false,
      port: 8895,
      headers: { "Access-Control-Allow-Origin": "*" },
      // to-do: proxy
      proxy: projectInfo.proxy || {}
    },
    mode: NODE_ENV,
    module: {
      rules: [
        // All output '.js' files will  re-processed by 'source-map-loader' and 'eslint-loader'.
        {
          enforce: "pre",
          test: /\.jsx?$/,
          use: [
            "source-map-loader",
            {
              loader: "eslint-loader",
              options: {
                emitWarning: true,
                failOnWarning: false,
                // emitError: true,
                failOnError: true,
                configFile: path.resolve(__dirname, "./config/.eslintrc"),
                useEslintrc: false
              }
            }
          ],
          exclude: /(node_modules|bower_components)/
        },
        // for NOT css/less modules
        {
          test: /[^.module]\.(css|less)$/,
          use: [
            {
              loader: prodMode ? MiniCssExtractPlugin.loader : "style-loader"
            },
            {
              loader: "css-loader"
            },
            {
              loader: "postcss-loader",
              options: {
                ident: "postcss",
                plugins: [
                  require("autoprefixer")({
                    add: prodMode,
                    // react doesn't support ie < 11 anyway, so why css?
                    // 和babelrc中保持一致
                    browsers: [
                      ">0.25%",
                      "not dead",
                      "not ie <= 11",
                      "not op_mini all"
                    ]
                  })
                ]
              }
            },
            {
              loader: "less-loader",
              options: {
                // antd modify colors
                modifyVars: {
                  "primary-color": "#ff4852",
                  "link-color": "#ff4852",
                  "border-radius-base": "2px"
                },
                javascriptEnabled: true
              }
            }
          ]
        },
        // for css/less modules
        {
          test: /\.module\.(css|less)$/,
          use: [
            {
              loader: prodMode ? MiniCssExtractPlugin.loader : "style-loader"
            },
            {
              loader: "css-loader",
              options: {
                modules: true
              }
            },
            {
              loader: "postcss-loader",
              options: {
                ident: "postcss",
                plugins: [
                  require("autoprefixer")({
                    add: prodMode,
                    // react doesn't support ie < 11 anyway, so why css?
                    browsers: [
                      ">0.25%",
                      "not dead",
                      "not ie <= 11",
                      "not op_mini all"
                    ]
                  })
                ]
              }
            },
            {
              loader: "less-loader"
            }
          ]
        },
        {
          test: /\.(sa|sc)ss$/,
          use: [
            {
              loader: prodMode ? MiniCssExtractPlugin.loader : "style-loader"
            },
            {
              loader: "css-loader"
            },
            {
              loader: "postcss-loader",
              options: {
                ident: "postcss",
                plugins: [
                  require("autoprefixer")({
                    add: prodMode,
                    // react doesn't support ie < 11 anyway, so why css?
                    browsers: [
                      ">0.25%",
                      "not dead",
                      "not ie <= 11",
                      "not op_mini all"
                    ]
                  })
                ]
              }
            },
            {
              loader: "sass-loader"
            }
          ]
        },
        // All files with a '.ts' or '.tsx' extension will be handled by 'awesome-typescript-loader'.
        {
          test: /\.tsx?$/,
          use: [
            {
              loader: "awesome-typescript-loader",
              options: {
                // 优先使用外部的tsconfig.json
                configFileName: externalTsConfigExist
                  ? path.resolve(basePath, "./tsconfig.json")
                  : path.resolve(__dirname, "./config/tsconfig.json")
              }
            }
          ]
        },
        // normal babel-loader
        {
          test: /\.(js|jsx)$/,
          exclude: prodMode
            ? /__invalidFolder/
            : /(node_modules|bower_components)/,
          loader: [
            {
              loader: "babel-loader",
              options: require(path.resolve(__dirname, "./config/.babelrc.js"))
            }
          ]
        },
        {
          test: /\.(jpg|gif|woff|woff2|eot|ttf|png|svg)/,
          use: [
            {
              loader: "url-loader",
              options: {
                // or handled by webpack as normal pic/asset
                limit: 100000
              }
            }
          ]
        }
        // {
        //   test: /\.svg/,
        //   use: [
        //     {
        //       loader: "url-loader",
        //       options: {
        //         limit: 100000,
        //         minetype: "image/svg+xml"
        //       }
        //     }
        //   ]
        // }
      ]
    },
    plugins: [
      new webpack.DefinePlugin({
        "process.env": {
          NODE_ENV: JSON.stringify(NODE_ENV)
        }
      }),
      new HtmlWebpackPlugin({
        filename: "index.html",
        template: path.resolve(basePath, "./index.ejs"),
        minify: {
          removeAttributeQuotes: prodMode,
          removeComments: prodMode,
          // minifyCSS: prodMode, // already minified
          minifyJS: prodMode
        },
        chunks: "all",
        // inlineSource: prodMode ? ".css$" : "",
        environment: NODE_ENV
      })
    ].concat(
      prodMode
        ? [
            new CleanWebpackPlugin(path.resolve(basePath, "./output"), {
              allowExternal: true
            }),
            new MiniCssExtractPlugin({
              filename: prodMode ? "app.[hash:8].css" : "[name].css",
              chunkFilename: prodMode ? "[id].[hash:8].css" : "[id].css"
            }),
            // will conflict with something...
            // ------------------------------------
            // Long Term Caching
            // ------------------------------------
            // More information https://medium.com/webpack/predictable-long-term-caching-with-webpack-d3eee1d3fa31
            new webpack.NamedChunksPlugin(chunk => {
              if (chunk.name) {
                return chunk.name;
              } else {
                // console.log(chunk);
                // debugger;
                // return String(++i);
                // 原来的代码会报错, 这里先hack一下...
                return [...chunk._modules][0].context.replace(/^\/.*\//, "");
              }

              // eslint-disable-next-line no-underscore-dangle
              // return [...chunk._modules]
              //   .map(m =>
              //     path.relative(
              //       m.context,
              //       m.userRequest.substring(0, m.userRequest.lastIndexOf('.'))
              //     )
              //   )
              //   .join('_');
            }),
            new webpack.HashedModuleIdsPlugin(),
            // ------------------------------------
            // Inject resulting css file into html...(optional)
            // ------------------------------------
            // new HtmlWebpackInlineSourcePlugin(),
            ifAnalyze
              ? new BundleAnalyzerPlugin({
                  port: "9981"
                })
              : () => {}
            // new StyleExtHtmlWebpackPlugin()
            // new CopyWebpackPlugin([
            //   {
            //     from: path.join(__dirname, './public'),
            //     to: path.join(__dirname, 'output/public')
            //   }
            // ])
          ]
        : []
    ),
    optimization: Object.assign(
      {
        splitChunks: {
          cacheGroups: {
            // ------------------------------------
            // this is for multi-entries only
            // ------------------------------------
            // commons: {
            //   name: 'commons',
            //   chunks: 'initial',
            //   minChunks: 2
            // },
            // ------------------------------------
            // vendors chunk
            // ------------------------------------
            vendors: {
              test(module) {
                // debugger
                // console.log(module, chunks);
                return (
                  /[\\/]node_modules[\\/]/.test(module.context) &&
                  !/[\\/]node_modules[\\/](react|react-dom)[\\/]/.test(
                    module.context
                  )
                );
              },
              name: "vendors",
              chunks: "all"
            },
            // ------------------------------------
            // react and react-dom chunk
            // ------------------------------------
            react: {
              test(module) {
                return /[\\/]node_modules[\\/](react|react-dom)[\\/]/.test(
                  module.context
                );
              },
              name: "react",
              chunks: "all"
            }
          }
          // styles: {
          //   name: 'styles',
          //   test: /\.css$/,
          //   chunks: 'all',
          //   enforce: true
          // }
        }
      },
      prodMode
        ? {
            minimizer: [
              new UglifyJsPlugin({
                cache: true,
                parallel: true,
                sourceMap: true // set to true if you want JS source maps
              }),
              new OptimizeCSSAssetsPlugin({})
            ]
          }
        : {}
    )
  };
};
